####################################################
#
#ESJ63 W16
#自由集会2016：メタバーコーディング・環境DNAバーコーディング解析の技法
#Last Version March 2016
#Kohmei Kadowaki
#Graduate School of Human & Environmental Studies, Kyoto University
#
####################################################

rm(list=ls(all=TRUE))

library(vegan) #必要なパッケージの読み込み
library(bipartite)
library(VGAM)
library(lars)

##############
#
#データセットの読み込み
#
##############

comm1<-read.table("table.txt",header=T) #群集行列データ（サンプルxOTUのリード数データ）を読み込む
idlist1<-read.delim("taxonomy.txt",header=T) #OTUの同定結果のデータを読み込む

taxon1<- idlist1[match(colnames(comm1),idlist1$query),] #群集行列に出現するOTUのみを選定する
masterdata<- cbind(t(comm1),taxon1) #群集行列と同定結果のデータを統合する

##############
#
# Rarefaction(希釈化)
#
##############

##############
#option 1 リード数が飽和したサンプルのみを用いる方法
##############

rarecurve(comm1[,-1]) # サンプルごとのリード数飽和度合いを可視化
rareReads<- 200 # 足切りを行うリード数を決定する
rarefiedcomm<- rrarefy(comm1[,-1],rareReads) # rarefactionを適用した行列を生成する
rarefiedcomm <- replace(rarefiedcomm,rarefiedcomm>0,1) #在不在行列に変換する


##############
#option2 リード数をカバレッジで揃える方法 (Chao and Jost 2012を参照)
##############

rareslopelist<-list()
for(i in 1:nrow(comm1)){rareslopelist[[i]]<-rareslope(comm1[i,-1],1:sum(comm1[i,-1]))}
cvr<-0.020 #揃えるrarefaction curveの傾きを設定 ; 傾き0.02=カバレッジ98%
cvrfun<-function(x){max(which(abs(x-cvr)<0.003))} #カバレッジに最も近いリード数を決める関数を設定 (ここでは、例として、誤差0.003の範囲で最大値を選択しているが他の設定方法も可能であり、rarefactionの結果を大きく変えることはない)
cvrrare<-unlist(lapply(rareslopelist,cvrfun)) #個々のサンプルについてカバレッジに最も近いリード数を計算
cvrno<-which(is.finite(cvrrare)) #カバレッジベースのrarefactionを適用されるサンプル番号を選定
plot(rowSums(comm1[cvrno,-1]),cvrrare[cvrno],xlab="Total No of reads", ylab="Rarefied No of reads") #サンプルあたり合計リード数とカバレッジで揃えたリード数の関係をプロット

#rarefaction curveの傾きを計算する関数を定義する
rarederatk <-
function (x, k)
{
   J <- sum(x)
   d <- digamma(J-k+1) -digamma(J-x-k+1)
   g <- lgamma(J-x+1) + lgamma(J-k+1) - lgamma(J-x-k+1)-lgamma(J+1)
   d <- d*exp(g)
   sum(d)
}

#rarefaction curveを描く
rarecurve(comm1[,-1],label=F,col=rgb(0, 0, 1, alpha=0.2),las=1)

#カバレッジで揃えたリード数水準を各サンプルについて赤い破線で図示する
y<-c();b<-c()
for(i in 1:length(cvrrare[cvrno]))
{
 y[i] <- rarefy(comm1[cvrno[i],-1], cvrrare[cvrno][i])
 b[i] <- rarederatk(comm1[cvrno[i],-1], cvrrare[cvrno][i])
 abline(y[i]-cvrrare[cvrno][i]*b[i], b[i],col=rgb(1, 0, 0, alpha=0.2))
 segments(cvrrare[cvrno][i],y[i],cvrrare[cvrno][i],0,col=rgb(1, 0, 0, alpha=0.2),lty=3)
}

covraredat<-data.frame() #カバレッジで揃えたrarefactionを適用する
for(i in 1:length(cvrno))
{covraredat<-rbind(covraredat,rrarefy(comm1[cvrno[i],-1],cvrrare[cvrno[i]]))}

covraredat<-replace(covraredat,covraredat>0,1) #在不在行列に変換する
rownames(covraredat)<-comm1[cvrno,1]

visweb(as.matrix(covraredat),type="none",box.border="gray") #行列の図示

##############
#
# α多様性、β多様性の計算と群集類似度指数
#
##############

#例として、Dinophyceaeに属するOTUのグループのみを抽出してデータ解析を行う
otuset<- match(idlist1[idlist1$class=="Dinophyceae",]$query,colnames(covraredat))
subcovraredat <- covraredat[,otuset]

#実験区画（調査地）のデータを生成
sitenames<-rep(c("Site_A", "Site_B"),each=8)

alpha <- rowSums(subcovraredat) #サンプルごとの種多様性(OTU数)を計算

beta<- betadisper(vegdist(subcovraredat,"jaccard"),sitenames) #β多様性を計算 (PERMDISP)
av.beta<-tapply(beta$distances,sitenames,mean) #サイトごとの平均β多様性を計算

mod<- metaMDS(vegdist(subcovraredat,"jaccard")) # 多次元尺度構成法 (NMDS)
plot(mod,display="sites",type="text")
ordihull(mod, sitenames, display = "sites", draw = "polygon", col="lightpink", border = "red",label=T) # Site A とBの群集組成をMDSにより視覚化した図

##############
#
# PERMANOVA & PERMDISP (土居・岡村2011も参照)
#
##############

#site Aとsite BにおけるDinophycea群集構造の違いを検定する PERMANOVA
permanova_ja<- adonis(vegdist(subcovraredat,"jaccard")~sitenames, permutations = 9999) #サイト間の群集構造の違いを検定する(PERMANOVA ; Anderson 2001 Austral Ecol)
permanova_rc<- adonis(raupcrick(subcovraredat)~sitenames, permutations = 9999)  #Raup-Crick index (Chase et al. 2011)を用いてβ多様性を計算する場合； Raup-Crick indexは、サンプルごとの種数の違いを補正した種組成の違いを定量化する指数である。

#同分散性の検定 (site A vs site B) PERMDISP
permutest(betadisper(vegdist(subcovraredat,"jaccard"),sitenames))


##############
#
# メタ個体群動態の解析
#
##############

#時間t＝1における行列をcomm1、時間t＝2における行列をcomm2と定義する
comm2<-read.table("table2.txt",header=T) #群集行列データ（サンプルxOTUのリード数データ）を読み込む

rareslopelist2<-list() #comm2についてもrarefactionを適用するため、curveの傾きを計算する
for(i in 1:nrow(comm2)){rareslopelist2[[i]]<-rareslope(comm2[i,-1],1:sum(comm2[i,-1]))}
cvrrare2<-unlist(lapply(rareslopelist2,cvrfun)) #個々のサンプルについてカバレッジに最も近いリード数を計算
cvrno2<-which(is.finite(cvrrare2))
covraredat2<-data.frame() #カバレッジで揃えたrarefactionを適用する
for(i in 1:length(cvrno2))
{covraredat2<-rbind(covraredat2,rrarefy(comm2[cvrno2[i],-1],cvrrare2[cvrno2[i]]))}
covraredat2<-replace(covraredat2,covraredat2>0,1) #在不在行列に変換する
rownames(covraredat2)<-comm2[cvrno2,1]


bivoddsdata<-data.frame() #各OTUについてt=1,2における在不在に基づく2x2行列を作成する
for(i in 1:ncol(covraredat))
 {
  bivoddsdata<- rbind(bivoddsdata,c(table(covraredat[,i],covraredat2[,i])))
 }
colnames(bivoddsdata)<- c("Y00", "Y01", "Y10", "Y11")

functgroup<- idlist1[match(idlist1$query,colnames(covraredat)),]$class #OTUのclass情報を抽出
functgroup<- relevel(functgroup,ref="Dinophyceae") #Dinophyceaeをベースラインとする
fit<-vglm(as.matrix(bivoddsdata) ~functgroup, binom2.or(zero = NULL),trace = TRUE) #二変量オッズ比の解析
coef(fit, matrix = TRUE)

#リンク関数の変更、xijによる回帰方程式の平行性の変更、t=1と2における環境変数をニッチ軸とするモデリングなど、vglmを活用することで柔軟なモデリングが可能となる；詳細はYee and Dirnboek (2009), Yee (2015) を参照

##############
#
# Checkerboard score (種間排他的分布パターンの検証)
# 排他的分布を生み出す要因として、ニッチ分割、競争排除、生息場所の選好性の違いなどがあげられる
#
##############

######
# step 1. Dinophycea群集全体における排他的分布のパターンを検証する
######

Cscore <- C.score(subcovraredat)

Ctest1<- oecosimu(subcovraredat, nestfun=C.score, method= "r00", nsimul=4999, burnin=100, alt = "greater") #合計の在数・不在数を固定し帰無モデルを生成
Ctest2<- oecosimu(subcovraredat, nestfun=C.score, method= "r0", nsimul=4999, burnin=100, alt = "greater") #サンプルごとの合計在数を固定し帰無モデルを生成
Ctest3<- oecosimu(subcovraredat, nestfun=C.score, method= "c0", nsimul=4999, burnin=100, alt = "greater") #OTUごとの合計在数を固定し帰無モデルを生成
Ctest4<- oecosimu(subcovraredat, nestfun=C.score, method= "swap", nsimul=4999, burnin=100, alt = "greater") #swapアルゴリズムで帰無モデルを生成

par(mfrow=c(2,2))

hist(Ctest1$oecosimu$simulated, xlab="C score", main="Scenario i (r00)") #帰無モデルから予測されたC scoreのヒストグラムを描く
abline(v=Cscore,col="red", lwd=2) #観測されたC scoreの値を表示する

hist(Ctest2$oecosimu$simulated, xlab="C score", main="Scenario ii (r0)")
abline(v=Cscore,col="red", lwd=2)

hist(Ctest3$oecosimu$simulated, xlab="C score", main="Scenario iii (c0)")
abline(v=Cscore,col="red", lwd=2)

hist(Ctest4$oecosimu$simulated, xlab="C score", main="Scenario iv (swap)")
abline(v=Cscore,col="red", lwd=2)


######
# step 2. Dinophyceaの各OTUの組み合わせにおける排他的分布パターンを検証する
######

Cscorefun<-function(x) {C.score(x,normalise = TRUE, FUN=c)} #全てのOTUペアについてCscoreを計算する関数を定義 (normalizeしているので0~1の値をとる)

ObservedCscoreset<- Cscorefun(subcovraredat) ##OTUの全てのペアについてCscoreを計算
Cscore_Pairwise<- combn(colnames(subcovraredat),2) #OTUの全組合わせを計算

Nsimul<- 4999 # 生成する帰無行列の数を指定する
simulatedatset<-permatswap(subcovraredat, method = "quasiswap",  fixedmar = "both", shuffle = "both", times=Nsimul) #帰無モデルデータの生成 ; optionとしてstrataによりデータの層別化を考慮することも可能

NullCscoreset<- matrix(unlist(lapply(simulatedatset$perm,Cscorefun)),Nsimul,) #生成された行列からOTUペアごとのCscoreを計算し整える（それぞれの列に同じペアの帰無モデルから計算されたCscoreが並ぶように整える）

par(mfrow=c(5,11),mar=c(2,2,2,2)) #全てのOTUペアについてCscoreと帰無モデルから期待されるCscoreを比較して図示する
for(i in 1:55)
 {
	hist(NullCscoreset[,i],axes=F,main=c(paste(Cscore_Pairwise[,i])),cex.main=0.5)
	abline(v=ObservedCscoreset[i],col="red", lwd=2)
 }

quantilefun<- function (x) {quantile(x, c(0.001,0.01,0.05,0.1, 0.25,0.5, 0.75, 0.9, 0.95, 0.99, 0.999))}
quantileset<- apply(NullCscoreset,2,quantilefun)
p.values <- quantileset[rownames(quantileset)=="95%",] #qunatile =0.95を有意水準に指定

p.advalues<- p.adjust(p.values, method = "BH") #Benjamini-Hochberg補正 (Benjamini & Hochberg 1995)

Cscore_Pairwise[,which(ObservedCscoreset-p.advalues>0)] #統計的に有意なC scoreを示すOTUのペアを表示

##############
#
# Lasso推定を用いた解析
#
##############

#どのOTUが一次生産性に貢献しているのかについて解析する場合を想定
subcovraredat<- as.matrix(subcovraredat) #Dinophyceaの群集データを行列に変換
primprod<- c(9.456134, 9.028364, 9.733087, 8.377099, 17.228074, 16.478002, 6.196396, 6.851176, 18.236788, 11.849765, 14.160797,  8.898115, 9.482257, 9.704055, 8.824272, 8.095436) #次世代シーケンスとは別に収集された一次生産性のデータなど、群集の機能に関するデータ（模擬データ）

lassofit<- lars(x=subcovraredat,y=primprod,type="lasso") #lasso推定
lambda<- lassofit$lambda #正則化パラメータの値
beta<- lassofit$beta #推定された回帰係数の値
Cp<- lassofit$Cp #推定されたCp値

layout(matrix(c(1,1,2,1,1,2),,2)) #lasso推定解析の結果
plot(c(min(lambda),max(lambda)),c(min(beta),max(beta)),xlab="λ",type="n",ylab="Estimated coefficients") #正則化パラメータを変化させ取り込まれる回帰係数セットを図示
for(i in 1:ncol(subcovraredat)){lines(lambda, beta[-1,i],bg=i, pch=21, type="p")}
for(i in 1:ncol(subcovraredat)){lines(lambda, beta[-1,i],col=i, lty=1)}
plot(Cp[-1]~lambda,ylim=c(min(Cp),max(Cp)),type="b",lty=1, xlab="λ",ylab="Cp") #正則化パラメータを変化させCpを図示


##############
#
#参考文献など
#
##############
# Anderson MJ (2001) A new method for non- parametric multivariate analysis of variance. Austral Ecology 26:32–46.
# Anderson MJ (2006) Distance-based tests for homo- geneity of multivariate dispersions. Biometrics 62:245–253.
# 安道知寛 (2014) 高次元データ分析の方法. 朝倉書店.
# Benjamini Y, Hochberg Y (1995) Controlling the false discovery rate: a practical and powerful approach to multiple testing. J. Roy. Statist. Soc. Ser. 1995, B57:289-300.
# Chao A, Jost L (2012) Coverage-based rarefaction and extrapolation: standardizing samples by completeness rather than size. Ecology 93:2253-2547
# Chase JM, Kraft NJB, Smith KG, Vellend M, Inouye BD (2011) Using null models to disentan- gle variation in community dissimilarity from variation in a-diversity. Ecosphere 2:24.
# 土居秀幸, 岡村寛 (2011) 生物群集解析のための類似度とその応用 : Rを使った類似度の算出、グラフ化、検定. 日本生態学会誌 61, 3-20
# Dormann et al. (2014) bipartite: Visualising bipartite networks and calculating some (ecological) indices.
# Gotelli NJ, Graves GR (1993) Null model ecology. Smithsonian Institution Press.
# Hastie T (2013) lars: Least Angle Regression, Lasso and Forward Stagewise.
# Oksanen J et al. (2015) vegan: Community Ecology Package.
# Stone L, Roberts A (1990) The checkerboard score and species distributions. Oecologia 85, 74–79
# 柳川尭 (1986) 離散多変量データの解析. 共立出版.
# van Belle G (2008) Statistical rules of thumb. Wiley Series in Probability and Statistics.
# Yee TW and Dirnboek T (2009) Models for analysing species presence/absence data at two time points. Journal of Theoretical Biology. 259, 684-694
# Yee TW (2015) Vector Generalized Linear and Additive Models. Springer Series in Statistics.
